/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import java.util.ArrayList;
import java.util.List;

public enum Metrics {
  
  /**
   * WARNING if you edit this file do not forget to also change sonar-web/src/main/rails/app/models/metric.rb
   */
 
  COBERTURA_LINE_COVERAGE( "cobertura_line_rate", false),
  COBERTURA_BRANCH_COVERAGE("cobertura_branch_rate", false),
  
  CLOVER_COVERAGE("clover_code_coverage", false),
  
  NCSS_PACKAGES("packages_count", false),
  NCSS_CLASSES("classes_count", false),
  NCSS_NCSS("ncss", true),
  NCSS_FUNCTIONS("functions_count", false),

  JDEPEND_TOTAL_CLASSES("totalClasses", false),
  JDEPEND_CONCRETE_CLASSES("concreteClasses", false),
  JDEPEND_ABSTRACT_CLASSES("abstractClasses", false),
  JDEPEND_AFFERENT_COUPLINGS("ca", false),
  JDEPEND_EFFERENT_COUPLINGS( "ce", false),
  JDEPEND_ABSTRACTNESS("a", false),
  JDEPEND_INSTABILITY( "i", false),
  JDEPEND_DISTANCE("d", false),
  JDEPEND_VOLATILITY("v", false),
  JDEPEND_PACKAGE_DEPENDENCY_CYCLES("packageCycle", false),

  PMD_DUPLICATION("duplication", false),
  PMD_DUPLICATED_LINES("duplicated_lines", true),
  PMD_DUPLICATED_TOKENS("duplicated_tokens", false),
  
  SUREFIRE_ERRORS("test_errors_count", false),
  SUREFIRE_SKIPPED("test_skipped_count", false),
  SUREFIRE_FAILURES("test_failures_count", false),
  SUREFIRE_TESTS("test_count", true),
  SUREFIRE_TIME("test_execution_time", true),
  SUREFIRE_SUCCESS_PERCENTAGE("test_success_percentage", false),
  SUREFIRE_ERRORS_PERCENTAGE("test_errors_percentage", false),
  SUREFIRE_FAILURE_PERCENTAGE("test_failure_percentage", false),

  CHANGELOG_COMMITS("commits", true),
  CHANGELOG_FILE_COMMITS("fileCommits", true),
  
  CK_WMC("wmc", false),
  CK_DIT("dit", false),
  CK_NOC("noc", false),
  CK_CBO("cbo", false),
  CK_RFC("rfc", false),
  CK_LCOM("lcom", false),
  CK_CA("ca", false),
  CK_NPM("npm", false),
  
  CYCLOMATIC_COMPLEXITY("ccn", true),
  CYCLOMATIC_COMPLEXITY_AVG_CLASS("ccn_class", true),
  CYCLOMATIC_COMPLEXITY_AVG_FUNCTION("ccn_function", true),
  COMPLEXITY_CLASSES_COUNT_DISTRIBUTION("ccn_classes_count_distribution", false),
  COMPLEXITY_CLASSES_PERCENT_DISTRIBUTION("ccn_classes_percent_distribution", false),
  
  CODE_COVERAGE("code_coverage", true),

  ERROR_RULES_INDEX("rules_compliance", true),
  ERROR_RULES_COUNT("rules_violations", true),
  WARNING_RULES_INDEX("optional_rules_compliance", true),
  WARNING_RULES_COUNT("optional_rules_violations", true),
  FAILURE_RULES_INDEX("strict_rules_compliance", true);
  
  private String name;
  private boolean tendencyAble;


  Metrics( String name, boolean tendencyAble ) {
    this.name = name;
    this.tendencyAble = tendencyAble;
  }


  public String getName() {
    return name;
  }
  
  public static Metrics[] tendencyableMetrics() {
    List<Metrics> tendencyable = new ArrayList<Metrics>();
    for ( Metrics metric : Metrics.values() ) {
      if ( metric.tendencyAble ) {
        tendencyable.add(metric);
      }
    }
    Metrics[] metrics = new Metrics[tendencyable.size()];
    return tendencyable.toArray(metrics);
  }
}
