/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import ch.hortis.sonar.model.MavenProject;
import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.model.Metrics;
import ch.hortis.sonar.model.ProjectMeasure;
import ch.hortis.sonar.model.RulesCategory;
import ch.hortis.sonar.model.Snapshot;

public class ProjectMeasureService extends Service {
  
  private static int ruleCategoriesCount = -1;
  
  public ProjectMeasureService( EntityManager manager ) {
    super( manager );
  }
  
  protected static synchronized int getRuleCategoriesCount( EntityManager manager ) {
    if (ruleCategoriesCount == -1) {
      ruleCategoriesCount = manager.createNamedQuery(RulesCategory.SQL_SELECT_ALL).getResultList().size();
    }
    return ruleCategoriesCount;
  }
  
  public Map<MeasureKey, List<ProjectMeasure>> getHistory(MavenProject project, Metrics[] metrics, java.util.Date startFrom, int maxResults) {
    MetricService metricsService = new MetricService(manager);
    List<Metric> targets = new ArrayList<Metric>();
    for (Metrics target : metrics) {
      targets.add(metricsService.getMetric(target));
    }

    Query query = manager.createNamedQuery(ProjectMeasure.SQL_SELECT_HISTORY);
    query.setMaxResults(getMaxHistoryReturnedRecords(metrics, maxResults));
    query.setParameter("project", project);
    query.setParameter("metrics", targets);
    query.setParameter("date", startFrom);
    query.setParameter("status", Snapshot.STATUS_PROCESSED);

    HashMap<MeasureKey, List<ProjectMeasure>> result = new HashMap<MeasureKey, List<ProjectMeasure>>();
    List<ProjectMeasure> measures = query.getResultList();
    for(ProjectMeasure measure : measures) {
      MeasureKey key = new MeasureKey(measure);
      List<ProjectMeasure> sublist = result.get(key);
      if (sublist == null) {
        sublist = new ArrayList<ProjectMeasure>();
        result.put(key, sublist);
      }
      if (sublist.size() < maxResults) {
        sublist.add(measure);
      }
    }
    return result;
  }

  protected int getMaxHistoryReturnedRecords(Metrics[] metrics, int maxResults) {
    int maxRes = maxResults * metrics.length;
    maxRes += maxResults * ( metrics.length * getRuleCategoriesCount(manager) );
    return maxRes;
  }
  
}
